/*
  1wire_1.c
                  1-Wire Communication Protocol
  05-06-2011                                                       */
//---I N C L U D E S-------------------------------------------------
#include "1wire_1.h"
//---V A R I A B L E S-----------------------------------------------
unsigned char macro_delay;
//-------------------------------------------------------------------
  #define wait(a) _asm movlw a\
	movwf macro_delay,1\
  loop:	nop\
        nop\
        nop\
        nop\
        nop\
        nop\
        decfsz macro_delay,1,1\
				bra loop\
  _endasm
//-------------------------------------------------------------------
void drive_OW_low (void)
{	OW_PIN_DIRECTION = OUTPUT;
	OW_WRITE_PIN=LOW;
}
//-------------------------------------------------------------------
void drive_OW_high (void)
{	OW_PIN_DIRECTION = OUTPUT;  //LATBbits.LATB0=0. 
	OW_WRITE_PIN = HIGH;	      //TRISBbits.TRISB0 1.
}
//-------------------------------------------------------------------
unsigned char read_OW (void)
{	unsigned char read_data=0;
	OW_WRITE_PIN = INPUT;      //TRISBbits.TRISB0 1.
  if (HIGH == OW_READ_PIN)   //1=PORTBbits.RB0. 
	 	read_data = SET;         //SET 1.
  else 	
		read_data = CLEAR;       //CLEAR 0.
	return read_data;
}
//-------------------------------------------------------------------
unsigned char OW_reset_pulse(void)
{	unsigned char presence_detect;
 	drive_OW_low(); 				     //Drive the bus low
 	wait(DELAY_240Us);	  			 //delay 480 microsecond (us)
	wait(DELAY_240Us);
 	drive_OW_high ();  				   //Release the bus
	wait(DELAY_70Us);				     //delay 70 microsecond (us)
	presence_detect = read_OW(); //Sample for presence pulse from slave
 	wait(DELAY_205Us);	  			 //delay 410 microsecond (us)
	wait(DELAY_205Us);		
	drive_OW_high ();		    	   //Release the bus
	return presence_detect;
}	
//-------------------------------------------------------------------
void OW_write_bit (unsigned char write_bit)
{ if (write_bit)
	{ //writing a bit '1'
		drive_OW_low(); 				// Drive the bus low
		wait(DELAY_6Us);				// delay 6 microsecond (us)
		drive_OW_high ();				// Release the bus
		wait(DELAY_64Us);				// delay 64 microsecond (us)
	}
	else
	{	//writing a bit '0'
		drive_OW_low(); 				// Drive the bus low
		wait(DELAY_60Us);				// delay 60 microsecond (us)
		drive_OW_high ();  				// Release the bus
		wait(DELAY_10Us);				// delay 10 microsecond for recovery (us)
	}
}	
//-------------------------------------------------------------------
unsigned char OW_read_bit (void)
{ unsigned char read_data; 
	//reading a bit 
	drive_OW_low(); 						// Drive the bus low
	wait(DELAY_6Us);						// delay 6 microsecond (us)
	drive_OW_high ();  					// Release the bus
	wait(DELAY_9Us);						// delay 9 microsecond (us)
	read_data = read_OW();			//Read the status of OW_PIN
	wait(DELAY_55Us);						// delay 55 microsecond (us)	
	return read_data;
}
//-------------------------------------------------------------------
void OW_write_byte (unsigned char write_data)
{	unsigned char loop;
	for (loop = 0; loop < 8; loop++)
	{	OW_write_bit(write_data & 0x01); 	//Sending LS-bit first
		write_data >>= 1;	//shift the data byte for the next bit to send
	}	
}	
//-------------------------------------------------------------------
unsigned char OW_read_byte (void)
{ unsigned char loop, result=0;
	for (loop = 0; loop < 8; loop++)
	{ result >>= 1; // shift the result to get it ready for the next bit to rec
		if (OW_read_bit())
		result |= 0x80;	// if result is one, then set MS-bit
	}
	return result;					
}	
