/**********************************************************
(c) 2006 Mauro Laurenti
All rights reserved
www.laurtec.com

 This library contains all the functions that are handy for the controlling
 of an LCD with a 44780 Hitachi controller
 This header file has been tested with 20MHz Quartz. You have to change the Delay
 function whether you are using a different Quartz.

 Available Functions

void   OpenLCD ()
void   ClearLCD ()
void   CursorLCD (char,char)		//1=ONcursor 0=OFF cursor;  1=ON blinking 0=OFF Blinking
void   WriteCharLCD (char)          //
void   GotoLineLCD (char line)      //Sposta il cursore sulle linee 14
void   LineLCD(char);
void   HomeLCD ()
void   Line2LCD();
void   ShiftLCD (char)				0=left	1=right
void   ShiftCursorLCD (char);			0=left	1=right
void   WriteStringLCD (" Hello World ")
void   WriteVarLCD(char *buffer)

void   SpostaCurLCD(char,char);     //Sposta il cursore DX/SX di n posizione

 to use these functions for your pourpose you must set up properly
 the following LCD constants and the pins of the PORT/s that you will use.
 You can use any pin you want!

***********************************************************
*/

#ifndef FLAG_LCD_44780
#define FLAG_LCD_44780

//*********************************************************
// LCD constants

#define LCD_D0 LATBbits.LATB4	// you must  set this pin as output
#define LCD_D1 LATBbits.LATB5	// you must  set this pin as output
#define LCD_D2 LATBbits.LATB6	// you must  set this pin as output
#define LCD_D3 LATBbits.LATB7	// you must  set this pin as output
#define LCD_RS LATCbits.LATC2	// you must  set this pin as output2
#define LCD_E  LATCbits.LATC1	// you must  set this pin as output1

//44*******************************************************
//Other Constants

#define LEFT 0
#define RIGHT 1
//Prototype
void HomeLCD(void);
void Epulse(void);
void ClearLCD(void);
void OpenLCD(void);
void Line2LCD(void);
//void Line3LCD(void); //Posiziona il cursore all'inizio riga 3.
//void Line4LCD(void); //Posiziona il cursore all'inizio riga 4.
void ShiftLCD(char L_R);
void ShiftCurDxLCD(unsigned char pos);   //Sposta il cursore a destra di pos.
void ShiftCurSxLCD(unsigned char pos);   //Sposta il cursore a sinistra di pos.
void SpostaCurLCD(char d_s,char n_pos);  //Sposta il cursore DX/SX di n posizioni.
void GotoLineLCD (char line);            //Cambia linea: 1,2,3,4.
//60*******************************************************
//this function accept in input the number of us for the dalay
//the number of nop inside the loop are trimmed for 20MHz Quartz

void delayLCD (int attesa)
{	int i;
	for (i =0; i<attesa;i++)
		{
			_asm
			nop
			nop
			nop
			nop
			_endasm
		}
}

//77*******************************************************
//This function create a pulse on the Pin Enable of the LCD
//to allow the sending of the command

void Epulse ()
{
	LCD_E = 1;           //83
	delayLCD (350);
	LCD_E = 0;           //85
	delayLCD (100);
}

//89*******************************************************
//Send a command. The input are the 8 command bits splitted in two 4 bit command
//You have to send the 4 high bit and than the 4 low bit

void SendCommand (unsigned char D3, unsigned char D2, unsigned char D1, unsigned char D0)
{
	LCD_D0 = D0;
	LCD_D1 = D1;
	LCD_D2 = D2;
	LCD_D3 = D3;
	Epulse ();
	delayLCD (20);
}

//***Posiziona il cursore all'inizio della riga 2**********
void Line2LCD()
{
	SendCommand (1,1,0,0);	// the cursor is moved to the second line
	SendCommand (0,0,0,0);
}

//***Posiziona il cursore all'inizio della riga 1**********

void HomeLCD()
{
	SendCommand (0,0,0,0);	// the cursor is moved at the beginning of the LCD
	SendCommand (0,0,1,0);
}

//****Cambia linea*****************************************
void GotoLineLCD (char line)
{   switch(line)
    {   case 1: SendCommand(1,0,0,0);
                SendCommand(0,0,0,0);
                break;
        case 2: SendCommand(1,1,0,0);
                SendCommand(0,0,0,0);
                break;
        case 3: SendCommand(1,0,0,1);
                SendCommand(0,1,0,0);
                break;
        case 4: SendCommand(1,1,0,1);
                SendCommand(0,1,0,0);
	}
}


//*********************************************************

void ShiftLCD(char L_R)
{
	SendCommand (0,0,0,1);	// the LCD is shifted on the left or on the right
	SendCommand (1,L_R,0,0);
}


//*********************************************************

void ShiftCursorLCD(char L_R)
{
	SendCommand (0,0,0,1);	// the cursor is shifted on the left or on the right
	SendCommand (0,L_R,0,0);
}

//---Sposta il cursore DX/SX di n posizioni.
void SpostaCurLCD(char d_s,char n_pos)
{   char i1,i2;
    for(i1=0;i1<n_pos-1;i1++)
    {  SendCommand (0,0,0,1);	// the cursor is shifted on the left or on the right
	      SendCommand (0,d_s,0,0);
       
    }

}

//---Posiziona il cursore all'inizio della riga 3----------
//void Line3LCD(void)
//{
//	unsigned char i;
//	HomeLCD();
//	for(i=0;i<20;i++)
//	{
//		ShiftCursorLCD(1);
//	}
//}
//---Posiziona il cursore all'inizio della riga 4----------
//void Line4LCD(void)
//{
//	unsigned char i;
//	Line2LCD();
//	for(i=0;i<20;i++)
//	{
//		ShiftCursorLCD(1);
//	}
//}
//---Posiziona il cursore a destra di N posizioni----------
void ShiftCurDxLCD(unsigned char pos)
{
	unsigned char i;
	for(i=0;i<pos;i++)
	{
		ShiftCursorLCD(1);
	}
}
//---Posiziona il cursore a sinistra di N posizioni--------
void ShiftCurSxLCD(unsigned char pos)
{
	unsigned char i;
	for(i=0;i<pos;i++)
	{
		ShiftCursorLCD(0);
	}
}
//---------------------------------------------------------
void ritardo(unsigned short long rit)
{
	unsigned short long i;
	for(i=0;i<rit;i++)
	{
	}
}

//*********************************************************
// write a character to the LCD

void WriteCharLCD (char Element)
{
	unsigned char D3,D2,D1,D0;

		LCD_RS = 1;

		D3 = (Element & 0b10000000) >> 7;	// Splitting of the first nibble
		D2 = (Element & 0b01000000) >> 6;
		D1 = (Element & 0b00100000) >> 5;
		D0 = (Element & 0b00010000) >> 4;

		SendCommand (D3,D2,D1,D0);

		D3 = (Element & 0b00001000) >> 3;	// Splitting of the second nibble
		D2 = (Element & 0b00000100) >> 2;
		D1 = (Element & 0b00000010) >> 1;
		D0 = (Element & 0b00000001);

		SendCommand (D3,D2,D1,D0);

		LCD_RS = 0;
}

//*********************************************************
// This function allow to write a costant string to the LCD

void WriteStringLCD(const rom char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer);  // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}

//*********************************************************
// This function allow to write a string Variable to the LCD

void WriteVarLCD(char *buffer)
{
        while(*buffer)                  // Write data to LCD up to null
        {
                WriteCharLCD(*buffer); // Write character to LCD
                buffer++;               // Increment buffer
        }
        return;
}
//*********************************************************
// this function clean the LCD

void ClearLCD ()
{
	SendCommand (0,0,0,0);
	SendCommand (0,0,0,1);
}

//*********************************************************
//This function turn OFF or ON the cursor of the display and its blinking
//0=Turn OFF	1= Turn ON

void CursorLCD(char ON_OFF,char Blink)
{
	SendCommand (0,0,0,0);
	SendCommand (1,1,ON_OFF,Blink);

}

//*********************************************************
//Reset the LCD

void  OpenLCD()
{
	// you have to set the port of the LCD properly

	LCD_RS = 0x00;
	LCD_E = 0x00;

	delayLCD (1000);
	SendCommand (0,0,1,1);
	delayLCD (1000);		// delay is more than 4mS
	SendCommand (0,0,1,1);
	delayLCD (1000);		// delay is more than 4mS
	SendCommand (0,0,1,1);
	delayLCD (1000);		// delay is more than 4mS
	SendCommand (0,0,1,0);
	SendCommand (0,0,1,0);
	SendCommand (1,0,0,0);
	SendCommand (0,0,0,0);
	SendCommand (1,1,1,0);
	CursorLCD (0,0);
	ClearLCD ();
}


#endif
